@@ -8,9 +8,12 @@ import numpy as np
 import pandas as pd
 import os, glob
 import datetime, warnings, requests
+from .exceptions import MinimizationError, InputError, DkeyWarning, IOWarning
+from .containers import hdict, vdict
 
 from mudpy import mdata
 from mudpy.containers import mdict, mvar, mhist
+from iminuit import Minuit
 
 __doc__="""
     Beta-data module. The bdata object is largely a data container, designed to 
@@ -25,10 +28,10 @@ __doc__="""
     Example usage ------------------------------------------------------------
     
         import bdata as bd
-        b = bd.bdata(40001)                 # read run 40001 from current year. 
+        b = bd.bdata(40001)                  # read run 40001 from current year. 
         b = bd.bdata(40001, year=2009)       # read run 40001 from year 2009.
-        b = bd.bdata(0, filename='file.msr') # read file from local memory, run 
-                                              number unused.
+        b = bd.bdata(filename='file.msr')    # read file from local memory, run 
+                                               number unused.
     Methods ------------------------------------------------------------------
     
         b.asym()            # calculate asymmetry. See bdata.asym docstring.     
@@ -137,10 +140,30 @@ class bdata(mdata):
             
             "Custom var enabled"                :"customv_enable", 
             "Custom var read name"              :"customv_name_read",    
-            "Custom var write name"             :"customv_name_write",    
+            "Custom var write name"             :"customv_name_write",   
+             
             "Start custom scan"                 :"customv_scan_start",    
             "Stop custom scan"                  :"customv_scan_stop",    
             "Custom Increment"                  :"customv_scan_incr", 
+            
+            "scan device"                       :"scan_device",
+            
+            "scan start"                        :"scan_start",
+            "scan stop"                         :"scan_stop",
+            "scan incr"                         :"scan_incr",
+            "scan increment"                    :"scan_incr",
+            
+            "NaVolt start (volts)"              :"scan_start", 
+            "NaVolt stop (volts)"               :"scan_stop", 
+            "NaVolt inc (volts)"                :"scan_incr", 
+            
+            "laser start (volts)"               :"scan_start", 
+            "laser stop (volts)"                :"scan_stop", 
+            "laser inc (volts)"                 :"scan_incr", 
+           
+            "field start (gauss)"               :"scan_start", 
+            "field stop (gauss)"                :"scan_stop", 
+            "field inc (gauss)"                 :"scan_incr", 
                                                         
             "DAQ drives sampleref"              :"smpl_ref_daq_drive", 
             "DAQ service time (ms)"             :"service_t", 
@@ -159,16 +182,14 @@ class bdata(mdata):
             "frequency start (Hz)"              :"freq_start", 
             "frequency stop (Hz)"               :"freq_stop", 
             
+            "scan var histo factor"             :"scan_var_histo_factor",
+            
             "init mode file"                    :"init_mode", 
             "init mode"                         :"init_mode", 
                                                         
             "helicity flip sleep (ms)"          :"hel_sleep", 
             "Helicity flip sleep(ms)"           :"hel_sleep", 
                                                                                                     
-            "NaVolt start (volts)"              :"volt_start", 
-            "NaVolt stop (volts)"               :"volt_stop", 
-            "NaVolt inc (volts)"                :"volt_incr", 
-                                                        
             "num bins"                          :"nbins", 
             "num cycles per supercycle"         :"ncycles", 
             "Number dwelltimes per freq"        :"ndwell_per_f", 
@@ -210,7 +231,9 @@ class bdata(mdata):
             "Single tone simulated"             :"sgle_tone_sim", 
                                                         
             "use defaults for midbnmr"          :"defaults", 
-             
+            
+            "probe species"                     :"probe_species",
+ 
         # CAMP
             "/biasV/input1"                             :"rb_cell_bias_set", 
             "/biasV/output1"                            :"rb_cell_bias_read", 
@@ -250,6 +273,7 @@ class bdata(mdata):
             "/Magnet/mag_set"                           :"b_field_setpt",     
             "/Magnet/mag_read"                          :"mag_current", 
             "/Magnet/controls/sys_status"               :"mag_ctrl_status", 
+            "/Magnet/ramp_status"                       :"mag_ramp_status", 
             "/Magnet/volts"                             :"mag_voltage", 
             "/mass_flow/read_flow"                      :"mass_read", 
             "/mass_flow/set_flow"                       :"mass_set",   
@@ -272,6 +296,7 @@ class bdata(mdata):
             "/Sample/current_read_1"                    :"smpl_current", 
             "/Sample/current_read"                      :"smpl_current", 
             "/Sample1/current_read"                     :"smpl_current", 
+            "/Sample/sample_read"                       :"smpl_read_A", 
             "/Sample/read_A"                            :"smpl_read_A", 
             "/Sample1/read_A"                           :"smpl_read_A", 
             "/Sample/read_B"                            :"smpl_read_B", 
@@ -355,7 +380,50 @@ class bdata(mdata):
             "":""
             }
     
+    # mapping from old run attributes to new run attributes
+    old_attr = {  
+        # histogram names
+            'FREQ'          :'Frequency', 
+            'Bp'            :'B+', 
+            'back+'         :'B+',
+            'Fp'            :'F+', 
+            'front+'        :'F+',
+            'Bm'            :'B-', 
+            'back-'         :'B-',
+            'Fm'            :'F-', 
+            'front-'        :'F-',
+            'FluM'          :'FluM2', 
+            'FluorMon2'     :'FluM2',
+            'PolLp'         :'L+', 
+            'PolLeft+'      :'L+',
+            'PolRp'         :'R+', 
+            'PolRight+'     :'R+', 
+            'PolLm'         :'L-',
+            'PolLeft-'      :'L-',
+            'PolRm'         :'R-', 
+            'PolRight-'     :'R-',
+            'NBMBp'         :'NBMB+', 
+            'NeutBmB+'      :'NBMB+',
+            'NBMFp'         :'NBMF+', 
+            'NeutBmF+'      :'NBMF+',
+            'NBMBm'         :'NBMB-', 
+            'NeutBmB-'      :'NBMB-',
+            'NBMFm'         :'NBMF-', 
+            'NeutBmF-'      :'NBMF-',
+        
+        # method to mode conversion
+            'TI-Bnmr'       :'1f',
+            'TI-bNMR'       :'1f',
+            
+            'TD-Bnmr'       :'20',
+            'TD-bNMR'       :'20',
+
+        # apparatus
+            'bnmr2'         :'BNMR',
+            'bnmr'          :'BNMR',
+        }
     
+    # options for asymmetry calculation
     option = {  ''                      :'',
                 
                 'adif'                  :'alpha_diffusion',
@@ -434,6 +502,15 @@ class bdata(mdata):
                 'slope_helicity'        :'slope_helicity',
     }
     
+    # output keys for mode TI runs
+    mode1_dict = {  '1f': 'freq',
+                    '1w': 'xpar',
+                    '1n': 'mV',
+                    '1e': 'mA',
+                    '1c': 'camp',
+                    '1d': 'las',
+                    }
+    
     # set environment variable same to get data archive location
     # should point to something like
     # "/data1/bnmr/dlog/" on linbnmr2
@@ -442,9 +519,13 @@ class bdata(mdata):
     evar_bnqr = "BNQR_ARCHIVE"
     
     # ======================================================================= #
-    def __init__(self, run_number, year=None, filename=""):
+    def __init__(self, run_number=None, year=None, filename=""):
         """Constructor. Reads file, stores and sorts data."""
             
+        # check input
+        if run_number is None and filename == '':
+            raise InputError("Bad input, specify either run_number or filename")
+            
         # convert dkeys keys to lower case
         bdata.dkeys = {k.lower():i for k, i in self.dkeys.items()}
             
@@ -509,95 +590,103 @@ class bdata(mdata):
                 warnings.warn('Run %d (%d) not found '% (run_number, year)+\
                               'locally. Fetched and saved to %s '%directory+\
                               'from musr.ca.', 
-                              category=Warning)
+                              category=IOWarning)
                         
         # Open and read file
         super().__init__(filename)
         
         # cast histogram data to floats
-        for key, hist in self.hist.items():
-            self.hist[key].data = hist.data.astype(np.float64)
-        
+        if hasattr(self, 'hist'):
+            for key, hist in self.hist.items():
+                self.hist[key].data = hist.data.astype(np.float64)
+            
         # Sort independent variables into dictionaries by title
-        self.ppg = mdict()
-        self.camp = mdict()
-        self.epics = mdict()
+        self.ppg = vdict()
+        self.camp = vdict()
+        self.epics = vdict()
         
         if hasattr(self, 'ivar'):
             
             for v in self.ivar.values(): 
-                try:
-                    if 'PPG' in v.title:
-                        self.ppg[bdata.dkeys[v.title.split("/")[-1].lower()]] = v
-                    elif v.title[0] == "/":
-                        self.camp[bdata.dkeys[v.title.lower()]] = v
-                    else:
-                        self.epics[bdata.dkeys[v.title.lower()]] = v
-                except (KeyError, IndexError):
-                        message = '"%s" not found in dkeys ("%s" in "%s"). ' +\
-                                    "Data in list, but not sorted to dict."
-                        message = message % (v.title, v.description, v.units)
-                        warnings.warn(message, RuntimeWarning, stacklevel=2)
+                if 'PPG' in v.title:
+                    title = v.title.split("/")[-1].lower()
+                    self.ppg[bdata.dkeys.get(title, title)] = v
+                elif v.title == "":
+                    pass
+                elif v.title[0] == "/":
+                    title = v.title.lower()
+                    self.camp[bdata.dkeys.get(title, title)] = v
+                else:
+                    title = v.title.lower()
+                    self.epics[bdata.dkeys.get(title, title)] = v
+                    
+                if title not in bdata.dkeys.keys():
+                        message = '%d.%d: "%s" not found in dkeys ("%s").'
+                        message = message % (self.year, self.run, v.title, v.description)
+                        warnings.warn(message, DkeyWarning, stacklevel=2)
             
-        # Add missing run mode for old runs
-        if year < 2005 and not self.mode and self.method == 'TI-bNMR':
-            self.mode = '1f' 
+        # Fix attributes for old runs
+        if year < 2005:
+            if not self.mode: 
+                self.mode = self.old_attr.get(self.method, self.method)
+            self.apparatus = self.old_attr.get(self.apparatus, self.apparatus)
             
-        # Fix inconsistent area for old runs
-        if year == 2003 and self.area == 'ISAC':
-            if self.run >= 45000:
-                self.area = 'BNQR'
-            else:
-                self.area = 'BNMR'
+            # histogram titles
+            hist = hdict()
             
-        # Fix histogram titles for old runs
-        if year == 2003 and 'FREQ' in self.hist.keys(): 
-            keymap = {'FREQ':'Frequency', 
-                      'Bp':'B+', 
-                      'Fp':'F+', 
-                      'Bm':'B-', 
-                      'Fm':'F-', 
-                      'FluM':'FluM2', 
-                      'PolLp':'L+', 
-                      'PolRp':'R+', 
-                      'PolLm':'L-', 
-                      'PolRm':'R-', 
-                      'NBMBp':'NBMB+', 
-                      'NBMFp':'NBMF+', 
-                      'NBMBm':'NBMB-', 
-                      'NBMFm':'NBMF-', 
-                      }
-            self.hist = mdict({i:self.hist[k] for k, i in keymap.items()})
             for k in self.hist.keys():
-                self.hist[k].title = k
-            
-        # prevent overwriting of attributes
-        self._bdata_initialised = True
-
+                newk = self.old_attr.get(k, k)
+                hist[newk] = self.hist[k]
+                hist[newk].title = newk
+            self.hist = hist
+        
+        # Fix inconsistent area for old runs
+        if self.area == 'ISAC':
+            if self.run >= 45000:   self.area = 'BNQR'
+            else:                   self.area = 'BNMR'
+    
     # ======================================================================= #
     def __getattr__(self, name):
         
-        try:
-            # fetch from top level
+        if name in ('hist', 'camp', 'ppg', 'epics'):
             return getattr(object, name)
-        except AttributeError as err:
-            
+        
+        else:
+
+            # fetch from top level
+            try:
+                return getattr(object, name)
+                
             # fetching of second level
-            if hasattr(self.camp, name): return getattr(self.camp, name)
-            if hasattr(self.epics, name):return getattr(self.epics, name)
-            if hasattr(self.ppg, name):  return getattr(self.ppg, name)
-            if hasattr(self.hist, name): return getattr(self.hist, name)
-                    
-            # nothing worked - raise error
-            raise AttributeError(err) from None
+            except AttributeError as err:
+                if hasattr(self, 'camp') and hasattr(self.camp, name):
+                    return getattr(self.camp, name)
+                if hasattr(self, 'epics') and hasattr(self.epics, name): 
+                    return getattr(self.epics, name)
+                if hasattr(self, 'ppg') and hasattr(self.ppg, name): 
+                    return getattr(self.ppg, name)
+                if hasattr(self, 'hist') and hasattr(self.hist, name): 
+                    return getattr(self.hist, name)
                         
+                raise AttributeError(err) from None
+                                
     # ======================================================================= #
     def __repr__(self):
         """
             Nice printing of parameters.
         """
         
+        # get all attributes
         d = self.__dict__
+        
+        # add beam properties
+        try: 
+            d['beam_keV'] = self.beam_keV
+            d['beam_keV_err'] = self.beam_keV_err
+            d['pulse_s'] = self.pulse_s
+        except AttributeError: 
+            pass
+        
         dkeys = list(d.keys())
         if dkeys:
             items = []
@@ -610,7 +699,7 @@ class bdata(mdata):
                     items.append([key, d[key].__class__])                
                     
                 # non iterables and mdict objects
-                elif not hasattr(d[key], '__iter__') or d[key].__class__ == mdict:
+                elif not hasattr(d[key], '__iter__') or d[key].__class__ in (mdict, vdict, hdict):
                     items.append([key, d[key]])                
                 
                 # strings
@@ -629,19 +718,85 @@ class bdata(mdata):
             return self.__class__.__name__ + "()"
         
     # ======================================================================= #
-    def __setattr__(self, name, value):
-        """Allow setting attributes only when initializing"""
+    def _beam_kev(self, get_error=False):
+        """
+            Get the beam energy in kev, based on typical biases: 
+                itw (or ite bias) - bias15 - platform bias
+                
+            if get_error: fetch error in value, rather than value
+        """
         
-        # this test allows attributes to be set in the __init__ method
-        if '_bdata__initialised' not in self.__dict__.keys(): 
-            return dict.__setattr__(self, name, value)
+        # get epics pointer
+        epics = self.epics
+        
+        # fetch stds
+        if get_error:   attr = 'std'
+        else:           attr = 'mean'
+        
+        # get inital beam energy in keV
+        beam = getattr(epics.target_bias, attr)/1000.
             
-        # any normal attributes are handled normally
-        elif name in self.__dict__.keys():       
-            raise AttributeError('Object is readonly')
+        # get Rb cell voltage
+        bias15 = getattr(epics.bias15, attr)/1000.
+        
+        # get platform bias 
+        if self.area.upper() == 'BNMR':
+            platform = getattr(epics.nmr_bias, attr)
+        elif self.area.upper() == 'BNQR':
+            platform = getattr(epics.nqr_bias, attr)/1000.
         else:
-            dict.__setattr__(self, name, value)
+            raise RuntimeError('Area not recognized')
+        
+        if get_error:
+            return np.sqrt(np.sum(np.square((beam, bias15, platform)))) # keV
+        else:
+            return beam-bias15-platform # keV
     
+    # ======================================================================= #
+    def _correct_deadtime(self, d, deadtime):
+        """
+            Apply deadtime correction based on bnmrfit physical script named
+            bnmr_fdt.pcm written by Jay and Zaher in June 2005
+            
+            d:          list of arrays for each counter as a function of time
+            deadtime:   float 
+        """
+        
+        try:
+        
+            # dwell time
+            tdwell = self.ppg['dwelltime'].mean * 0.001
+             
+            # time to do one scan
+            if self.mode in ('20', '2h', '00'):
+                tscan = tdwell * (self.ppg['beam_on'].mean + \
+                                  self.ppg['beam_off'].mean + \
+                                  self.ppg['prebeam'].mean) \
+                        + self.ppg['hel_sleep'].mean * 0.001
+
+                # number of scans
+                nscans = self.duration / tscan
+
+            elif '1' in self.mode:
+                nscans = self.ppg['nbins'].mean
+
+        # missing PPG parameters
+        except KeyError:    
+            if deadtime > 0:
+                warnings.warn(
+                    "%d.%d: Missing PPG parameter(s), no deadtime correction applied" \
+                        % (self.year, self.run),
+                    Warning, stacklevel=2)
+            return d
+            
+        # time per bin
+        tbin = tdwell * nscans
+        
+        # apply correction 
+        dnew = [di / (1 - di/tbin*deadtime) for di in d]
+        
+        return dnew
+        
     # ======================================================================= #
     def _get_area_data(self, nbm=False):
         """Get histogram list based on area type. 
@@ -651,19 +806,19 @@ class bdata(mdata):
         
         hist = self.hist
         
-        if self.mode == '1n' or nbm:
+        if nbm:
             data = [hist['NBMF+'].data, \
                     hist['NBMF-'].data, \
                     hist['NBMB+'].data, \
                     hist['NBMB-'].data]
             
-        elif self.area == 'BNMR':
+        elif self.area.upper() == 'BNMR':
             data = [hist['F+'].data, \
                     hist['F-'].data, \
                     hist['B+'].data, \
                     hist['B-'].data]
         
-        elif self.area == 'BNQR':
+        elif self.area.upper() == 'BNQR':
             data = [hist['R+'].data, \
                     hist['R-'].data, \
                     hist['L+'].data, \
@@ -1024,6 +1179,8 @@ class bdata(mdata):
     # ======================================================================= #
     def _get_xhist(self):
         """Get histogram data for x axis."""
+        
+        # histogram name
         if self.mode == '1f':
             xlabel = 'Frequency'
         elif self.mode == '1w':
@@ -1034,8 +1191,18 @@ class bdata(mdata):
                     break
         elif self.mode == '1e':
             xlabel = 'Magnet mA'
+        elif self.mode == '1c':
+            xlabel = 'Camp'
+        elif self.mode == '1d':
+            xlabel = 'Laser'
         
-        return self.hist[xlabel].data
+        # get data
+        try:
+            return self.hist[xlabel].data
+            
+        # custom scan
+        except KeyError:
+            return self.hist['Custom EPICS scan'].data
     
     # ======================================================================= #
     def _rebin(self, xdx, rebin):
@@ -1066,19 +1233,20 @@ class bdata(mdata):
         
         # weighted mean
         for i in np.arange(0, lenx, rebin):
-            w = 1./dx[i:i+rebin-1]**2
+            w = 1./dx[i:i+rebin]**2
             wsum = np.sum(w)
             
             if wsum == 0:
-                x_rebin.append(np.mean(x[i:i+rebin-1]))
-                dx_rebin.append(np.std(x[i:i+rebin-1]))
+                x_rebin.append(np.mean(x[i:i+rebin]))
+                dx_rebin.append(np.std(x[i:i+rebin]))
             else:
-                x_rebin.append(np.sum(x[i:i+rebin-1]*w)/wsum)
+                x_rebin.append(np.sum(x[i:i+rebin]*w)/wsum)
                 dx_rebin.append(1./wsum**0.5)
+                
         return np.array([x_rebin, dx_rebin])
-            
+                 
     # ======================================================================= #
-    def asym(self, option="", omit="", rebin=1, hist_select='', nbm=False):
+    def asym(self, option="", omit="", rebin=1, hist_select='', nbm=False, deadtime=0):
         """Calculate and return the asymmetry for various run types. 
            
         usage: asym(option="", omit="", rebin=1, hist_select='', nbm=False)
@@ -1094,6 +1262,7 @@ class bdata(mdata):
                                 with [, ] or [;]. Histogram names cannot 
                                 therefore contain either of these characters.
             nbm:            if True, use neutral beams in calculations
+            deadtime:       detector deadtime used to correct counter values (s)
             
         Asymmetry calculation outline (with default detectors) ---------------
         
@@ -1137,11 +1306,12 @@ class bdata(mdata):
                      nTR is the right counter tagged with !alphas (absence of), 
                      nLR is the right counter tagged with !alphas, 
                                           
-                  
         Status of Data Corrections --------------------------------------------
             SLR/2H: 
                 Removes prebeam bins. 
                 
+                Provides the option of deadtime correction. Set to zero to disable.
+                
                 Rebinning: 
                     returned time is average time over rebin range
                     returned asym is weighted mean
@@ -1149,6 +1319,8 @@ class bdata(mdata):
             1F/1W: 
                 Allows manual removal of unwanted bins. 
                 
+                Provides the option of deadtime correction. Set to zero to disable.
+                
                 Scan Combination:
                     Multiscans are considered as a single scan with long 
                     integration time. Histogram bins are summed according to 
@@ -1159,13 +1331,14 @@ class bdata(mdata):
                     as Possionian.
             
             1N:
-                Same as 1F. Uses the neutral beam monitor values to calculate 
-                asymetries in the same manner as the NMR calculation. 
+                Same as 1F. Use the neutral beam monitor values to calculate 
+                asymetries in the case of 8Li. 
              
             2E: 
                 Prebeam bin removal. 
                 Postbeam bin removal. Assumes beamoff time is 0. 
                 Splits saved 1D histograms into 2D.
+                Deadtime corrections are not yet available for this mode. 
                 
                 Asymmetry calculations: 
                     raw: calculated through differences method (as described in 
@@ -1269,21 +1442,21 @@ class bdata(mdata):
         try:
             option = self.option[option]
         except KeyError:
-            raise RuntimeError("Option not recognized.")
+            raise InputError("Option not recognized.")
         
         # get data
         if hist_select != '':
             
             # split into parts
             hist_select_temp = []
-            for histname in hist_select.split(', '):
-                hist_select_temp.extend(histname.split(';'))
+            for histname in hist_select.split(','):
+                hist_select_temp.extend(histname.strip().split(';'))
             hist_select = [h.strip() for h in hist_select_temp]
             
             # check for user error
             if len(hist_select) < 4:
-                raise RuntimeError('hist_select must be a string of at least '+\
-                        'four [, ]-seperated or [;]-seperated histogram names')
+                raise InputError('hist_select must be a string of at least '+\
+                        'four [,]-seperated or [;]-seperated histogram names')
             
             # get data
             d = [self.hist[h].data for h in hist_select]
@@ -1293,14 +1466,14 @@ class bdata(mdata):
         else:
             d = self._get_area_data(nbm=nbm) # 1+ 2+ 1- 2-
             d_all = d
-            
-        # get alpha diffusion data
+                
+        # get alpha diffusion data and apply deadtime corrections
         if self.mode == '2h':
             d_alpha = d[4:]
             d = d[:4]
-        
+            
         # SLR -----------------------------------------------------------------
-        if self.mode in ("20", '2h'):
+        if self.mode in ("20", '2h', "00"):
             
             # get the number of prebeam bins
             try:
@@ -1334,6 +1507,13 @@ class bdata(mdata):
                     for i in range(len(d)):
                         d[i] = np.delete(d[i], [0])
                         
+                    if self.mode == '2h':
+                        for i in range(len(d_alpha)):
+                            d_alpha[i] = np.delete(d_alpha[i], [0])
+                        
+            # deadtime correction
+            d = self._correct_deadtime(d, deadtime)
+            
             # do alpha background subtractions
             if self.mode == '2h':    
                 for i in range(len(d_alpha)):
@@ -1352,7 +1532,7 @@ class bdata(mdata):
             if rebin > 1:
                 len_t = len(time)
                 new_t_idx = np.arange(0, len_t, rebin)
-                new_time = (np.average(time[i:i+rebin-1]) for i in new_t_idx)
+                new_time = (np.average(time[i:i+rebin]) for i in new_t_idx)
                 time = np.fromiter(new_time, dtype=float, count=len(new_t_idx))
 
             # mode switching
@@ -1391,7 +1571,7 @@ class bdata(mdata):
                     asym = self._get_asym_alpha(d_alpha, d)
                 except UnboundLocalError as err:
                     if self.mode != '2h':
-                        raise RuntimeError('Run is not in 2h mode.')
+                        raise InputError('Run is not in 2h mode.')
                 return np.vstack([time, self._rebin(asym, rebin)])
             
             elif option == 'alpha_tagged': # ---------------------------------
@@ -1399,7 +1579,7 @@ class bdata(mdata):
                     asym = self._get_asym_alpha_tag(d_alpha, d)  
                 except UnboundLocalError as err:
                     if self.mode != '2h':
-                        raise RuntimeError('Run is not in 2h mode.')
+                        raise InputError('Run is not in 2h mode.')
                     else:
                         raise err
                 
@@ -1438,17 +1618,13 @@ class bdata(mdata):
                 return out
         
         # 1F ------------------------------------------------------------------
-        elif self.mode in ('1f', '1n', '1w', '1e'):
+        elif self.mode in self.mode1_dict.keys(): # ('1f', '1n', '1w', '1e', '1c', '1d')
             
             # get xaxis label and data key
-            if self.mode == '1f':
-                xlab = 'freq'
-            elif self.mode == '1w':
-                xlab = 'xpar'
-            elif self.mode == '1n':
-                xlab = 'mV'
-            elif self.mode == '1e':
-                xlab = 'mA'
+            xlab = self.mode1_dict[self.mode]
+            
+            # deadtime correction
+            d = self._correct_deadtime(d, deadtime)
             
             # get bins to kill
             bin_ranges_str = further_options 
@@ -1579,75 +1755,118 @@ class bdata(mdata):
         else:
             print("Unknown run type.")
             return
-
+            
     # ======================================================================= #
     def beam_kev(self, get_error=False):
-        """
-            Get the beam energy in kev, based on typical biases: 
-                itw (or ite bias) - bias15 - platform bias
-                
-            if get_error: fetch error in value, rather than value
-        """
-        
-        # get epics pointer
-        epics = self.epics
-        
-        # fetch stds
-        if get_error:
-            attr = 'std'
-        else:
-            attr = 'mean'
-        
-        # get inital beam energy in keV
-        beam = getattr(epics.target_bias, attr)/1000.
-            
-        # get RB cell voltage
-        bias15 = getattr(epics.bias15, attr)/1000.
-        
-        # get platform bias 
-        if self.area == 'BNMR':
-            platform = getattr(epics.nmr_bias, attr)
-        elif self.area == 'BNQR':
-            platform = getattr(epics.nqr_bias, attr)/1000.
-        else:
-            raise RuntimeError('Area not recognized')
-        
-        if get_error:
-            return np.sqrt(np.sum(np.square((beam, bias15, platform)))) # keV
-        else:
-            return beam-bias15-platform # keV
+        warnings.warn('beam_kev() will be depreciated in the next major '+\
+                      'release. Use beam_keV or beam_keV_err properties instead.', 
+                      Warning)
+                      
+        if get_error:   return self.beam_keV_err
+        else:           return self.beam_keV
+    
+    @property
+    def beam_keV(self):     return self._beam_kev()
+    @property
+    def beam_keV_err(self): return self._beam_kev(get_error=True)
     
+    # ======================================================================= #
+    def get_deadtime(self, dt=1e-9, c=1, return_minuit=False, fixed='c'):
+        """
+            Get detector deadtime in s (TD mode only)
+            
+            Based on bnmrfit physical script named bnmr_fdt.pcm written by Jay 
+            and Zaher in June 2005
+            
+            dt:             deadtime initial parameter in s
+            c:              negative helicity scaling factor
+            return_minuit:  if true, return Minuit object, else return the 
+                            unfixed parameters
+            fixed:          if string fix that parameter, list fix those parameters
+                            
+            returns
+                
+                if return_minuit return Minuit object
+                if one parameter fixed, return the other parameter
+                if both parameters fixed, return the chi2 value
+        """
+        
+        # check run mode
+        if '2' not in self.mode:
+            raise RuntimeError('Deadtimes only estimatable in time differentiated mode')
+        
+        # make chi2 function: compare the midpoint of the split helicity
+        # to the total average value, which is somewhere in the middle
+        def chi(dt_ns, c):
+            
+            # get split hel asym
+            asym = self.asym('h', deadtime=dt_ns*1e-9)
+            p, dp = asym['p']
+            n, dn = asym['n']
+            
+            # midpoints
+            midpts = 0.5*(p+c*n)
+            dmidpts2 = 0.25*(dp**2 + (c*dn)**2)
+            
+            # weighted average midpoint
+            avgmid = np.average(midpts, weights=1/dmidpts2)
+            
+            # get chi2
+            return np.mean( ((midpts-avgmid)**2)/dmidpts2 )
+        
+        
+        # search for best chi2
+        m = Minuit(chi, dt_ns=dt*1e9, c=c)
+        m.errordef = 1
+        m.limits['dt_ns'] = (0, None)
+        
+        # fixing
+        if 'c' in fixed:    m.fixed['c'] = True
+        if 'dt' in fixed:   m.fixed['dt_ns'] = True
+        
+        # run minimization
+        m.migrad()
+        
+        # check if valid 
+        if not m.valid:
+            print(m.fmin)
+            print(m.params)
+            raise MinimizationError("Minuit failed to converge to a valid minimum")
+        
+        # make output
+        if return_minuit:   
+            return m
+        else:      
+            output = []
+            if not m.fixed['dt_ns']:
+                output.append(m.values['dt_ns']*1e-9)
+            if not m.fixed['c']:
+                output.append(m.values['c'])
+            
+            # return
+            len_out = len(output)
+            if len_out == 0:    return m.fval           # all fixed: return chi2
+            elif len_out == 1:  return output[0]
+            else:               return np.array(output)
+        
     # ======================================================================= #
     def get_pulse_s(self):
+        warnings.warn('get_pulse_s() will be depreciated in the next major '+\
+                      'release. Use pulse_s property instead.', Warning)
+        return self.pulse_s
+        
+    @property
+    def pulse_s(self):        
         """Get pulse duration in seconds, for pulsed measurements."""
         
         try:
             dwelltime = self._get_ppg('dwelltime')
+        except KeyError:
+            raise AttributeError("Missing ppg parameter: dwelltime") from None
+        try:
             beam_on = self._get_ppg('beam_on')
-        except AttributeError:
-            raise AttributeError("Missing logged ppg parameter: dwelltime "+\
-                    "or beam_on") from None
+        except KeyError:
+            raise AttributeError("Missing ppg parameter: beam_on") from None
+                    
         return dwelltime*beam_on/1000.
     
-# set lifetimes for various particles in seconds, with errors (ref: rmlm slr_v2.cpp)
-# commented out isotopes with multiple decay products not taken into account by bfit
-life = mdict({
-            "Li8"       :1.2096,    # http://journals.aps.org/prc/abstract/10.1103/PhysRevC.82.027309
-            "Li8_err"   :0.0005, 
-            "Li9"       :0.2572,    # http://journals.aps.org/prc/abstract/10.1103/PhysRevC.13.835
-            "Li9_err"   :0.0006, 
-            "Li11"      :0.0126,   # http://www.sciencedirect.com/science/article/pii/S0375947412000413
-            "Li11_err"  :0.0020, 
-            "Be11"      :19.85,     # http://www.sciencedirect.com/science/article/pii/S0375947412000413
-            "Be11_err"  :0.10,   
-            "F20"       :16.105,    # http://www.sciencedirect.com/science/article/pii/037594749290251E
-            "F20_err"   :0.012,  
-            "Mg29"      :1.88,     # http://www.sciencedirect.com/science/article/pii/S0375947403018074
-            "Mg29_err"  :0.17,   
-            "Mg31"      :0.332,     # http://www.sciencedirect.com/science/article/pii/S0375947403018074
-            "Mg31_err"  :0.029,   
-            "Ac230"     :175.76,    # http://isys01.triumf.ca/search/isotope/data/view?z=89&a=230&m=0
-            "Ac232"     :171.43,    # http://isys01.triumf.ca/search/isotope/data/view?z=89&a=232&m=0
-            "Ac234"     :63.49,     # https://www.sciencedirect.com/science/article/pii/0375947486900254
-            "Ac234_err" :10.1,      # https://www.sciencedirect.com/science/article/pii/0375947486900254
-        })
